# SPDX-FileCopyrightText: © 2025 DSLab - Fondazione Bruno Kessler
#
# SPDX-License-Identifier: Apache-2.0

from __future__ import annotations

from digitalhub.entities.model._base.spec import ModelSpec, ModelValidator


class ModelSpecSklearn(ModelSpec):
    """
    ModelSpecSklearn specifications.
    """


class ModelValidatorSklearn(ModelValidator):
    """
    ModelValidatorSklearn validator.
    """
