# SPDX-FileCopyrightText: © 2025 DSLab - Fondazione Bruno Kessler
#
# SPDX-License-Identifier: Apache-2.0

from __future__ import annotations

from digitalhub.entities.trigger._base.status import TriggerStatus


class TriggerStatusLifecycle(TriggerStatus):
    """
    TriggerStatusLifecycle status.
    """
