# SPDX-FileCopyrightText: © 2025 DSLab - Fondazione Bruno Kessler
#
# SPDX-License-Identifier: Apache-2.0

from __future__ import annotations

ANIMALS = [
    "ant",
    "bear",
    "bee",
    "bison",
    "cat",
    "cheetah",
    "cougar",
    "crane",
    "deer",
    "dolphin",
    "duck",
    "eagle",
    "falcon",
    "finch",
    "fox",
    "frog",
    "gecko",
    "hawk",
    "heron",
    "horse",
    "jaguar",
    "koala",
    "leopard",
    "lion",
    "llama",
    "lynx",
    "mole",
    "moose",
    "otter",
    "owl",
    "panda",
    "penguin",
    "rabbit",
    "robin",
    "seal",
    "shark",
    "sparrow",
    "tiger",
    "tortoise",
    "wolf",
]

ADJECTIVES = [
    "able",
    "agile",
    "alert",
    "alpha",
    "amber",
    "apex",
    "arctic",
    "atomic",
    "balanced",
    "basic",
    "bold",
    "bright",
    "brisk",
    "calm",
    "clever",
    "clear",
    "cloudy",
    "crisp",
    "curious",
    "cyber",
    "daring",
    "deep",
    "delta",
    "digital",
    "dynamic",
    "early",
    "easy",
    "electric",
    "epic",
    "fast",
    "firm",
    "fluent",
    "focused",
    "fresh",
    "friendly",
    "full",
    "future",
    "gentle",
    "global",
    "golden",
    "grand",
    "great",
    "green",
    "happy",
    "high",
    "honest",
    "hyper",
    "ideal",
    "infinite",
    "inner",
    "instant",
    "keen",
    "kind",
    "known",
    "large",
    "light",
    "linear",
    "logical",
    "lucky",
    "lunar",
    "magic",
    "major",
    "mega",
    "mild",
    "modern",
    "neat",
    "noble",
    "open",
    "outer",
    "peak",
    "perfect",
    "plain",
    "prime",
    "pure",
    "quick",
    "quiet",
    "rapid",
    "rare",
    "ready",
    "real",
    "red",
    "right",
    "safe",
    "sharp",
    "silent",
    "simple",
    "smart",
    "solid",
    "solar",
    "speedy",
    "stable",
    "steady",
    "stellar",
    "strong",
    "swift",
    "true",
    "ultra",
    "vast",
    "vital",
    "wise",
]
