# SPDX-FileCopyrightText: © 2025 DSLab - Fondazione Bruno Kessler
#
# SPDX-License-Identifier: Apache-2.0

from __future__ import annotations

from digitalhub.entities._base.material.status import MaterialStatus


class ArtifactStatus(MaterialStatus):
    """
    Status class for artifact entities.
    """


class ArtifactStatusArtifact(ArtifactStatus):
    """
    Status class for artifact entities.
    """
