# SPDX-FileCopyrightText: © 2025 DSLab - Fondazione Bruno Kessler
#
# SPDX-License-Identifier: Apache-2.0

from __future__ import annotations

from digitalhub.entities.trigger._base.spec import TriggerSpec, TriggerValidator


class TriggerSpecAutoml(TriggerSpec):
    """
    TriggerSpecAutoml specifications.
    """


class TriggerValidatorAutoml(TriggerValidator):
    """
    TriggerValidatorAutoml validator.
    """
