# SPDX-FileCopyrightText: © 2025 DSLab - Fondazione Bruno Kessler
#
# SPDX-License-Identifier: Apache-2.0

from __future__ import annotations

from digitalhub.entities.dataitem._base.spec import DataitemSpec, DataitemValidator


class DataitemSpecDataitem(DataitemSpec):
    """
    DataitemSpecDataitem specifications.
    """


class DataitemValidatorDataitem(DataitemValidator):
    """
    DataitemValidatorDataitem validator.
    """
