# SPDX-FileCopyrightText: © 2025 DSLab - Fondazione Bruno Kessler
#
# SPDX-License-Identifier: Apache-2.0
from digitalhub.stores.client.base.client import Client
from digitalhub.stores.client.base.factory import get_client
